﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.DataImporter.Data;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Command {
  [StorableClass]
  [ViewableCommandInfoAttribute("Previous Value", 1, ColumnGroupState.AnySelectedColumnContainsNull, "Handle Missing Values", Position = 3)]
  public class FillMissingValuesWithPreviousValuesCommand : FillMissingValueCommandBase {
    [StorableConstructor]
    protected FillMissingValuesWithPreviousValuesCommand(bool deserializing) : base(deserializing) { }
    public FillMissingValuesWithPreviousValuesCommand(DataSet dataSet, string columnGroupName, int[] affectedColumns) :
      base(dataSet, columnGroupName, affectedColumns) {
    }

    public override void Execute() {
      base.Execute();
      IComparable val = null;
      ColumnBase column;
      oldSortOrder = ColumnGroup.SortOrdersForColumns.ToList();
      oldSortedColumnIndexes = new List<int>(ColumnGroup.SortedColumnIndexes);
      foreach (int col in AffectedColumns) {
        column = ColumnGroup.GetColumn(col);
        if (column.ContainsNullValues) {
          if (column.SortOrder != SortOrder.None)
            ColumnGroup.ResetSorting();
          nullValues.Add(col, new List<int>());
          val = null;
          for (int j = 0; j < column.TotalValuesCount; j++) {
            if (column.GetValue(j) != null)
              val = column.GetValue(j);
            else {
              column.ChangeValue(j, val);
              this.nullValues[col].Add(j);
            }
          }
        }
      }
      ColumnGroup.FireChanged();
      ColumnGroup = null;
    }

    public override string Description {
      get { return "Fill missing values with previous value"; }
    }
  }
}
